@@ -84,6 +84,9 @@ class TimezoneComponent(Component):
         good_lines = ('rdate', 'rrule', 'dtstart', 'tzname', 'tzoffsetfrom',
                       'tzoffsetto', 'tzid')
         buffer = StringIO.StringIO()
+        # allow empty VTIMEZONEs
+        if len(self.contents) == 0:
+            return None
         def customSerialize(obj):
             if isinstance(obj, Component):
                 foldOneLine(buffer, u"BEGIN:" + obj.name)
@@ -267,6 +270,10 @@ class TimezoneComponent(Component):
         # try PyICU's tzid key
         if hasattr(tzinfo, 'tzid'):
             return tzinfo.tzid
+            
+        # try pytz zone key
+        if hasattr(tzinfo, 'zone'):
+            return tzinfo.zone
 
         # try tzical's tzid key
         elif hasattr(tzinfo, '_tzid'):
@@ -484,9 +491,11 @@ class RecurringComponent(Component):
                             values['BYMONTHDAY'] = [str(n) for n in rule._bymonthday]
 
                     if rule._bymonth is not None and len(rule._bymonth) > 0:
-                        if not (rule._freq == dateutil.rrule.YEARLY and
-                                len(rule._bymonth) == 1 and
-                                rule._bymonth[0] == rule._dtstart.month):
+                        if (rule._byweekday is not None or
+                            len(rrule._bynweekday or ()) > 0 or
+                            not (rule._freq == dateutil.rrule.YEARLY and
+                                 len(rule._bymonth) == 1 and
+                                 rule._bymonth[0] == rule._dtstart.month)):
                             # ignore bymonth if it's generated by dateutil
                             values['BYMONTH'] = [str(n) for n in rule._bymonth]
 
@@ -792,8 +801,8 @@ class VCalendar2_0(behavior.Behavior):
         findTzids(obj, tzidsUsed)
         oldtzids = [x.tzid.value for x in getattr(obj, 'vtimezone_list', [])]
         for tzid in tzidsUsed.keys():
-            if tzid == 'UTC' or tzid in oldtzids: continue
-            obj.add(TimezoneComponent(tzinfo=getTzid(tzid)))
+            if tzid != 'UTC' and tzid not in oldtzids:
+                obj.add(TimezoneComponent(tzinfo=getTzid(tzid)))
 registerBehavior(VCalendar2_0)
 
 class VTimezone(behavior.Behavior):
@@ -812,7 +821,11 @@ class VTimezone(behavior.Behavior):
 
     @classmethod
     def validate(cls, obj, raiseException, *args):
-        return True #TODO: FIXME
+        if not hasattr(obj, 'tzid') or obj.tzid.value is None:
+            if raiseException:
+                m = "VTIMEZONE components must contain a valid TZID"
+                raise ValidateError(m)
+            return False            
         if obj.contents.has_key('standard') or obj.contents.has_key('daylight'):
             return super(VTimezone, cls).validate(obj, raiseException, *args)
         else:
